from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.utils import timezone

from evotedocs.documents import PdfDocument

from reportlab.platypus import Indenter


class ElectoralProcessPdf(PdfDocument):

    def set_story(self, **kwargs):
        electoral_process = kwargs.get('electoral_process')

        if electoral_process.is_draft():
            self.story.append(self.title(_("Electoral Process creation")))
        else:
            self.story.append(self.title(_("Electoral Process") + " " +
                                         electoral_process.identifier))

        self.story.append(self.pure_sec_title(_("Summary")))
        self.story.append(self.space())
        self.story.append(self.pure_sec_title("1. " + _("General Data")))
        self.story.append(self.get_elecproc_table(electoral_process))

        # Adds all elections
        for i, election in enumerate(electoral_process.elections.all()):
            self.story.append(self.pure_sec_title(
                "1." + str(i + 1) + " " + _('General Data | Election ') +
                election.identifier))
            self.story.append(self.get_election_table(election))
            self.story.append(self.hr(space_after=10))
            self.story.append(self.label(_('Electoral timetable')))
            self.story.append(Indenter(15, 15))
            self.get_electoral_calendar(election)
            self.story.append(Indenter(-15, -15))

        self.story.append(self.pure_sec_title(
            "3. " + _("Electoral Comission")))
        self.story.append(self.space())
        self.story.append(Indenter(-15, 0))
        self.story.append(self.get_members_table(electoral_process))

    def get_elecproc_table(self, electoral_process):
        """ Returns the table with electoral process identification """
        image = ''
        if electoral_process.logo:
            image = self.image(electoral_process.logo)
        table_data = ([[
            '',
            self.label("Electoral process identifier"),
            self.label("Description")
        ], [image,
            self.body(electoral_process.identifier),
            self.body(electoral_process.description)]])

        return self.table(table_data, col_widths=[100, 200, 200] * 2)

    def get_election_table(self, election):
        """ Returns the table with election identification """
        data = ([[
            self.label(_("Election identifier")),
            self.label(_("Description"))
        ], [
            self.body(election.identifier),
            self.body(election.description)
        ]])

        return self.table(data)

    def get_electoral_calendar(self, election):
        """ Appends to story the electoral calendar of election """
        self.story.append(self.pure_sec_title(_("Proposition dates")))
        self.story.append(self.get_date_table(
            _("Begin date"), _("End date"),
            self.get_formated_date(election.proposition_begin_date),
            self.get_formated_date(election.proposition_end_date)))
        self.story.append(self.pure_sec_title(_("Electoral roll")))
        self.story.append(self.get_date_table(
            _("Publication date before verification"),
            _("Publication date after verification and correction"),
            self.get_formated_date(election.roll_begin_date),
            self.get_formated_date(election.roll_end_date)))
        self.story.append(self.pure_sec_title(_("Voting time")))
        self.story.append(self.get_date_table(
            _("Begin date"), _("End date"),
            self.get_formated_date(election.voting_begin_date),
            self.get_formated_date(election.voting_end_date)))
        self.story.append(self.pure_sec_title(_("Voting results")))
        self.story.append(self.get_date_table(
            _("Date of publication"),
            '',
            self.get_formated_date(election.minute_publication_date),
            ''))
        self.story.append(self.get_date_table(
            _("Approval period - start"),
            _("Approval period - end"),
            self.get_formated_date(election.minute_begin_date),
            self.get_formated_date(election.minute_end_date))
        )

    def get_date_table(self, id_one, id_two, field_one, field_two):
        """ Returns the table with electoral process identification """
        desc_one = self.body(_('GMT Timezone')) if field_one else ''
        desc_two = self.body(_('GMT Timezone')) if field_two else ''

        data = ([[self.label(id_one), '', self.label(id_two), ''],
                [field_one, desc_one, field_two, desc_two]])

        return self.table(data)

    def get_members_table(self, electoral_process):
        """ Returns the table of members """
        table_members = []
        table_members.append([
            self.label('#'),
            self.label(_('Name')),
            self.label(_('Role')),
            self.label(_('Designation')),
            self.label(_('Email address')),
            self.label(_('Contact Phone Number'))
        ])
        for i, member in enumerate(electoral_process.electoral_members.all()):
            table_members.append([
                self.body(str(i + 1)),
                self.body(member.user.get_full_name()),
                self.body(member.get_role_display()),
                self.body(member.designation),
                self.body(member.user.get_email()),
                self.body(str(member.user.get_phone()))
            ])

        return self.long_table(
            table_members, header=True,
            col_widths=[20, 100, 100, 100, 100, 100])

    def get_title(self, **kwargs):
        electoral_process = kwargs.get('electoral_process')
        return _("{} Summary".format(electoral_process))

    def set_footer(self, **kwargs):
        self.footer.append(self.hr(space_after=0))
        now = timezone.now().strftime('%d-%M-%Y %H:%M:%S')
        text = "{}: {}".format(_('Generated in'), now)
        self.footer.append(self.body(text))


class ElectionCountingsPdf(PdfDocument):

    def set_story(self, **kwargs):
        election = kwargs.get('election')
        table_data = election.count_queries.all()
        table = []
        self.story.append(
            self.title(_("Total votes in ballot box of election: ") +
                       election.identifier))
        self.story.append(self.pure_sec_title(""))

        self.story.append(self.votes_table(
            [[_('Number of votes in ballot box') + ":"], [10]]))

        table.append([
            self.label(_('Requested By')),
            self.label(_('Date')),
            self.label(_('# Votes')),
        ])
        for elem in table_data:
            table.append([
                self.body(str(elem.user.get_short_name())),
                self.get_formated_date(elem.created, icon=False),
                self.body(str(elem.total_count))
            ])
        self.story.append(self.long_table(table, header=True))

    def set_footer(self, **kwargs):
        self.footer.append(self.hr(space_after=0))
        now = timezone.now().strftime('%d-%M-%Y %H:%M:%S')
        text = "{}: {}".format(_('Generated in'), now)
        self.footer.append(self.body(text))
